<?php

/**
* Copyright © 2016 Magento. All rights reserved.
* See COPYING.txt for license details.
*/
namespace FiloBlu\Ldap\Model;

use Magento\Framework\Exception\Plugin\AuthenticationException as PluginAuthenticationException;
use Magento\Framework\Phrase;

/**
* Backend Auth model
*/
class Auth extends \Magento\Backend\Model\Auth
{

    protected static $ldaplink = null;
    public $host;
    public $version;
    public $rootDn;
    public $rootPassword;
    public $userDn;
    public $loginTemplate;
    public $filter;
    public $cmpAttr;
    public $pwdAttr;
    public $attr;
    public $tls;
    public $roleId;
    public $actived;
    public $data = array();
    public $roleTemplate = '';
    public $user_exists_with_new_password = false;
    public $port;
    protected $_helper;
    protected $_roles;
    protected $_userModel;
    protected $_messages;


    /**
     * @param \Magento\Framework\Event\ManagerInterface $eventManager
     * @param \Magento\Backend\Helper\Data $backendData
     * @param \Magento\Backend\Model\Auth\StorageInterface $authStorage
     * @param \Magento\Backend\Model\Auth\Credential\StorageInterface $credentialStorage
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $coreConfig
     * @param \Magento\Framework\Data\Collection\ModelFactory $modelFactory
     */
    public function __construct(
        \Magento\Framework\Event\ManagerInterface $eventManager,
        \Magento\Backend\Helper\Data $backendData,
        \Magento\Backend\Model\Auth\StorageInterface $authStorage,
        \Magento\Backend\Model\Auth\Credential\StorageInterface $credentialStorage,
        \Magento\Framework\App\Config\ScopeConfigInterface $coreConfig,
        \Magento\Framework\Data\Collection\ModelFactory $modelFactory,
        \FiloBlu\Ldap\Helper\Data $ldapData,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \Magento\Authorization\Model\ResourceModel\Role\Collection $roleCollection,
        \Magento\User\Model\User $user
    ) {
        $this->_eventManager = $eventManager;
        $this->_backendData = $backendData;
        $this->_authStorage = $authStorage;
        $this->_credentialStorage = $credentialStorage;
        $this->_coreConfig = $coreConfig;
        $this->_modelFactory = $modelFactory;
        $this->_helper = $ldapData;
        $this->_roles = $roleCollection;
        $this->_userModel = $user;
        $this->_messages = $messageManager;
    }

    private function load_Parameters(){
        // 'cn=admin,dc=diva,dc=com';
        $this->rootDn = trim($this->_coreConfig->getValue('dcadmin/ldaplogin/rootdn'));
        // '*******'
        $this->rootPassword =  $this->_coreConfig->getValue('dcadmin/ldaplogin/rootpassword');
        //'ou=users,dc=diva,dc=com'
        $this->userDn = trim($this->_coreConfig->getValue('dcadmin/ldaplogin/userdn'));
        // '(&(%s=%s)(groups=DiVA-magento-1))';
        $this->loginTemplate = trim($this->_coreConfig->getValue('dcadmin/ldaplogin/user_login_template'));
        //
        $this->filter = trim($this->_coreConfig->getValue('dcadmin/ldaplogin/filter'));
        // 'cn';
        $this->cmpAttr = trim($this->_coreConfig->getValue('dcadmin/ldaplogin/cmpattr'));
        // 'ldap1'
        $this->host = trim($this->_coreConfig->getValue('dcadmin/ldaplogin/host'));
        // '3'
        $this->version = intval(trim($this->_coreConfig->getValue('dcadmin/ldaplogin/version')));
        // '389'
        $this->port = intval(trim($this->_coreConfig->getValue('dcadmin/ldaplogin/port')));
        // false
        $this->tls = intval(trim($this->_coreConfig->getValue('dcadmin/ldaplogin/tls')));
        // cn,givenname,mail,sn,displayname,userpassword
        $this->attr = json_decode(trim($this->_coreConfig->getValue('dcadmin/ldaplogin/attr')), true);
        //default the role_id after each login 0 to disable
        $this->roleId = intval(trim($this->_coreConfig->getValue('dcadmin/ldaplogin/defaultroleid')));
        //actived 1|0
        $this->pwdAttr = trim($this->_coreConfig->getValue('dcadmin/ldaplogin/passattr'));
        //actived 1|0
        $this->actived = trim($this->_coreConfig->getValue('dcadmin/ldaplogin/activeldap'));
    }

    public function get_Link(){
        if(is_null(self::$ldaplink)) {
            $this->connect();
        }

        return self::$ldaplink;
    }

    private function connect(){
//        ldap_set_option(null, LDAP_OPT_DEBUG_LEVEL, 7);
        $this->load_Parameters();
        if (is_null(self::$ldaplink)){
            if ($this->tls) {
                $url = 'ldaps://'.$this->host.'/';
            } else {
                $url = 'ldap://'.$this->host.'/';
            }

            self::$ldaplink = ldap_connect($url, $this->port);
        }

        if (!self::$ldaplink) {
            return false;
        }

        if (!ldap_set_option(self::$ldaplink, LDAP_OPT_PROTOCOL_VERSION, $this->version)) {
            $this->_helper->log('AUTH_ADMIN ERROR : VERSION ERROR');
            return false;
        }

        ldap_set_option(self::$ldaplink, LDAP_OPT_REFERRALS, 0); // NB!!!

        if (!@ldap_bind(self::$ldaplink, $this->rootDn, $this->rootPassword)) {
            $this->_helper->log('AUTH_ADMIN ERROR : BIND ERROR. Could not connect to ldap');
            self::$ldaplink = null;
            return false;
        }

        return true;
    }

    /**
     * Perform login process
     *
     * @param string $username
     * @param string $password
     * @return void
     * @throws \Magento\Framework\Exception\AuthenticationException
     */
    public function login($username, $password)
    {
        if (empty($username) || empty($password)) {
            self::throwException(__('You did not sign in correctly or your account is temporarily disabled.'));
        }

        try {
            $this->load_Parameters();

            //IF DESACTIVED DO NORMAL AUTH
            if (!$this->actived) { //CHECK MAGENTO CONNECT
                return parent::login($username, $password);
            }

            // check that we have LDAP module installed
            if (!function_exists('ldap_connect')) {
                return parent::login($username, $password);
            }

            if (!$this->get_Link()) {
                return parent::login($username, $password);
            }

            $ldap_user = $this->authentify($username, $password);

            if (!$this->user_exists_with_new_password && $ldap_user)
            {
                $this->checkCreate($ldap_user,$password);
            }

            elseif ($this->user_exists_with_new_password)
                self::throwException(
                    __('You did not sign in correctly or your account is temporarily disabled.')
            );




            $this->_initCredentialStorage();
            $this->getCredentialStorage()->login($username, $password);
            if ($this->getCredentialStorage()->getId()) {
                $this->getAuthStorage()->setUser($this->getCredentialStorage());
                $this->getAuthStorage()->processLogin();

                $this->_eventManager->dispatch(
                    'backend_auth_user_login_success',
                    ['user' => $this->getCredentialStorage()]
                );
            }

            if (!$this->getAuthStorage()->getUser()) {
                self::throwException(__('You did not sign in correctly or your account is temporarily disabled.'));
            }
        } catch (PluginAuthenticationException $e) {
            $this->_eventManager->dispatch(
                'backend_auth_user_login_failed',
                ['user_name' => $username, 'exception' => $e]
            );
            throw $e;
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            $this->_eventManager->dispatch(
                'backend_auth_user_login_failed',
                ['user_name' => $username, 'exception' => $e]
            );
            self::throwException(
                __($e->getMessage()? : 'You did not sign in correctly or your account is temporarily disabled.')
            );
        }
    }

    public function authentify($login=null, $password=null){
        if (is_null($login) || is_null($password) || $login == '' || $password == '') {
            return false;
        }

        try{
            // try to bind to ldap to verify password
            $user_login = sprintf($this->loginTemplate, $login);
            if (@ldap_bind($this->get_Link(), $user_login, $password) === true) {
                if ($this->is_Allowed($login)) {
                    return $this;
                }
            } else {
                self::$ldaplink = null; // reset resource to reconnect. Avoids some errors
                $this->user_exists_with_new_password = $this->is_Allowed($login); // check if the user is somewhere there
            }
            $this->_messages->addError('no Group assigned');
            return false;
        }catch(Exception $e){
            self::$ldaplink = null; // reset resource to reconnect. Avoids some errors
            $this->user_exists_with_new_password = $this->is_Allowed($login); // check if the user is somewhere there
            return false;
        }
    }

    public function is_Allowed($login){
        $filter = sprintf($this->filter, $this->cmpAttr, $login);
        $searchDn = $this->userDn;
        $data = ldap_search($this->get_Link(), $searchDn, $filter, array_values($this->attr));
        if ($data) {
            $this->data = ldap_get_entries(self::$ldaplink, $data);
            if ($this->data['count'] == 1) {
                if ($this->data[0]['businesscategory']['count'] > 0) {
                    foreach ($this->data[0]['businesscategory'] as $index_member => $member_group) {
                        if ((string)$index_member != 'count' && strpos($member_group, 'GG_Magento-') !== false) {
                            $this->_roles->addFieldToFilter('role_type', 'G')->load();
                            foreach ($this->_roles as $role_object) {
                                $role_searchable_name = str_replace(' ', '', $role_object->getRoleName());
                                //$this->roleTemplate = 'CN=GG_Magento-%s,OU=groups,DC=filoblu,DC=com';
                                $this->roleTemplate = 'GG_Magento-%s';
                                if (mb_strtolower(sprintf($this->roleTemplate, $role_searchable_name)) == mb_strtolower($member_group)) {
                                    $this->roleId = $role_object->getId();
                                }
                            }
                        }
                    }
                }
            }
        }

        return $this->roleId != 0 ? true : false;

    }

    public function checkCreate($ldap_user,$password) {

        $username = $ldap_user->data[0]['cn'][0];

        $this->_userModel->loadByUsername($username);

        $this->_userModel->setUsername($username)
            ->setFirstname($ldap_user->data[0]['givenname'][0])
            ->setLastname($ldap_user->data[0]['sn'][0])
            ->setEmail($ldap_user->data[0]['mail'][0])
            ->setPassword($password)
            ->setRoleId($this->roleId)
            ->save();

        $this->_messages->addSuccess('User Data Updated');


    }


}